# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .invoices import (
    InvoicesResource,
    AsyncInvoicesResource,
    InvoicesResourceWithRawResponse,
    AsyncInvoicesResourceWithRawResponse,
    InvoicesResourceWithStreamingResponse,
    AsyncInvoicesResourceWithStreamingResponse,
)
from .payments import (
    PaymentsResource,
    AsyncPaymentsResource,
    PaymentsResourceWithRawResponse,
    AsyncPaymentsResourceWithRawResponse,
    PaymentsResourceWithStreamingResponse,
    AsyncPaymentsResourceWithStreamingResponse,
)

__all__ = [
    "PaymentsResource",
    "AsyncPaymentsResource",
    "PaymentsResourceWithRawResponse",
    "AsyncPaymentsResourceWithRawResponse",
    "PaymentsResourceWithStreamingResponse",
    "AsyncPaymentsResourceWithStreamingResponse",
    "InvoicesResource",
    "AsyncInvoicesResource",
    "InvoicesResourceWithRawResponse",
    "AsyncInvoicesResourceWithRawResponse",
    "InvoicesResourceWithStreamingResponse",
    "AsyncInvoicesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .country_code import CountryCode

__all__ = ["MiscListSupportedCountriesResponse"]

MiscListSupportedCountriesResponse: TypeAlias = List[CountryCode]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from .._models import BaseModel
from .discount_type import DiscountType

__all__ = ["Discount"]


class Discount(BaseModel):
    amount: int
    """The discount amount.

    - If `discount_type` is `percentage`, this is in **basis points** (e.g., 540 =>
      5.4%).
    - Otherwise, this is **USD cents** (e.g., 100 => `$1.00`).
    """

    business_id: str
    """The business this discount belongs to."""

    code: str
    """The discount code (up to 16 chars)."""

    created_at: datetime
    """Timestamp when the discount is created"""

    discount_id: str
    """The unique discount ID"""

    restricted_to: List[str]
    """List of product IDs to which this discount is restricted."""

    times_used: int
    """How many times this discount has been used."""

    type: DiscountType
    """The type of discount, e.g. `percentage`, `flat`, or `flat_per_unit`."""

    expires_at: Optional[datetime] = None
    """Optional date/time after which discount is expired."""

    name: Optional[str] = None
    """Name for the Discount"""

    usage_limit: Optional[int] = None
    """Usage limit for this discount, if any."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["Currency"]

Currency: TypeAlias = Literal[
    "AED",
    "ALL",
    "AMD",
    "ANG",
    "AOA",
    "ARS",
    "AUD",
    "AWG",
    "AZN",
    "BAM",
    "BBD",
    "BDT",
    "BGN",
    "BHD",
    "BIF",
    "BMD",
    "BND",
    "BOB",
    "BRL",
    "BSD",
    "BWP",
    "BYN",
    "BZD",
    "CAD",
    "CHF",
    "CLP",
    "CNY",
    "COP",
    "CRC",
    "CUP",
    "CVE",
    "CZK",
    "DJF",
    "DKK",
    "DOP",
    "DZD",
    "EGP",
    "ETB",
    "EUR",
    "FJD",
    "FKP",
    "GBP",
    "GEL",
    "GHS",
    "GIP",
    "GMD",
    "GNF",
    "GTQ",
    "GYD",
    "HKD",
    "HNL",
    "HRK",
    "HTG",
    "HUF",
    "IDR",
    "ILS",
    "INR",
    "IQD",
    "JMD",
    "JOD",
    "JPY",
    "KES",
    "KGS",
    "KHR",
    "KMF",
    "KRW",
    "KWD",
    "KYD",
    "KZT",
    "LAK",
    "LBP",
    "LKR",
    "LRD",
    "LSL",
    "LYD",
    "MAD",
    "MDL",
    "MGA",
    "MKD",
    "MMK",
    "MNT",
    "MOP",
    "MRU",
    "MUR",
    "MVR",
    "MWK",
    "MXN",
    "MYR",
    "MZN",
    "NAD",
    "NGN",
    "NIO",
    "NOK",
    "NPR",
    "NZD",
    "OMR",
    "PAB",
    "PEN",
    "PGK",
    "PHP",
    "PKR",
    "PLN",
    "PYG",
    "QAR",
    "RON",
    "RSD",
    "RUB",
    "RWF",
    "SAR",
    "SBD",
    "SCR",
    "SEK",
    "SGD",
    "SHP",
    "SLE",
    "SLL",
    "SOS",
    "SRD",
    "SSP",
    "STN",
    "SVC",
    "SZL",
    "THB",
    "TND",
    "TOP",
    "TRY",
    "TTD",
    "TWD",
    "TZS",
    "UAH",
    "UGX",
    "USD",
    "UYU",
    "UZS",
    "VES",
    "VND",
    "VUV",
    "WST",
    "XAF",
    "XCD",
    "XOF",
    "XPF",
    "YER",
    "ZAR",
    "ZMW",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel
from .currency import Currency
from .tax_category import TaxCategory

__all__ = ["AddonResponse"]


class AddonResponse(BaseModel):
    id: str
    """id of the Addon"""

    business_id: str
    """Unique identifier for the business to which the addon belongs."""

    created_at: datetime
    """Created time"""

    currency: Currency
    """Currency of the Addon"""

    name: str
    """Name of the Addon"""

    price: int
    """Amount of the addon"""

    tax_category: TaxCategory
    """Tax category applied to this Addon"""

    updated_at: datetime
    """Updated time"""

    description: Optional[str] = None
    """Optional description of the Addon"""

    image: Optional[str] = None
    """Image of the Addon"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["LicenseValidateParams"]


class LicenseValidateParams(TypedDict, total=False):
    license_key: Required[str]

    license_key_instance_id: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .customers import (
    CustomersResource,
    AsyncCustomersResource,
    CustomersResourceWithRawResponse,
    AsyncCustomersResourceWithRawResponse,
    CustomersResourceWithStreamingResponse,
    AsyncCustomersResourceWithStreamingResponse,
)
from .customer_portal import (
    CustomerPortalResource,
    AsyncCustomerPortalResource,
    CustomerPortalResourceWithRawResponse,
    AsyncCustomerPortalResourceWithRawResponse,
    CustomerPortalResourceWithStreamingResponse,
    AsyncCustomerPortalResourceWithStreamingResponse,
)

__all__ = [
    "CustomerPortalResource",
    "AsyncCustomerPortalResource",
    "CustomerPortalResourceWithRawResponse",
    "AsyncCustomerPortalResourceWithRawResponse",
    "CustomerPortalResourceWithStreamingResponse",
    "AsyncCustomerPortalResourceWithStreamingResponse",
    "CustomersResource",
    "AsyncCustomersResource",
    "CustomersResourceWithRawResponse",
    "AsyncCustomersResourceWithRawResponse",
    "CustomersResourceWithStreamingResponse",
    "AsyncCustomersResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..types import license_activate_params, license_validate_params, license_deactivate_params
from .._types import NOT_GIVEN, Body, Query, Headers, NoneType, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.license_key_instance import LicenseKeyInstance
from ..types.license_validate_response import LicenseValidateResponse

__all__ = ["LicensesResource", "AsyncLicensesResource"]


class LicensesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LicensesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return LicensesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LicensesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return LicensesResourceWithStreamingResponse(self)

    def activate(
        self,
        *,
        license_key: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LicenseKeyInstance:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/licenses/activate",
            body=maybe_transform(
                {
                    "license_key": license_key,
                    "name": name,
                },
                license_activate_params.LicenseActivateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LicenseKeyInstance,
        )

    def deactivate(
        self,
        *,
        license_key: str,
        license_key_instance_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            "/licenses/deactivate",
            body=maybe_transform(
                {
                    "license_key": license_key,
                    "license_key_instance_id": license_key_instance_id,
                },
                license_deactivate_params.LicenseDeactivateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def validate(
        self,
        *,
        license_key: str,
        license_key_instance_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LicenseValidateResponse:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/licenses/validate",
            body=maybe_transform(
                {
                    "license_key": license_key,
                    "license_key_instance_id": license_key_instance_id,
                },
                license_validate_params.LicenseValidateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LicenseValidateResponse,
        )


class AsyncLicensesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLicensesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLicensesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLicensesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return AsyncLicensesResourceWithStreamingResponse(self)

    async def activate(
        self,
        *,
        license_key: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LicenseKeyInstance:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/licenses/activate",
            body=await async_maybe_transform(
                {
                    "license_key": license_key,
                    "name": name,
                },
                license_activate_params.LicenseActivateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LicenseKeyInstance,
        )

    async def deactivate(
        self,
        *,
        license_key: str,
        license_key_instance_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            "/licenses/deactivate",
            body=await async_maybe_transform(
                {
                    "license_key": license_key,
                    "license_key_instance_id": license_key_instance_id,
                },
                license_deactivate_params.LicenseDeactivateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def validate(
        self,
        *,
        license_key: str,
        license_key_instance_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LicenseValidateResponse:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/licenses/validate",
            body=await async_maybe_transform(
                {
                    "license_key": license_key,
                    "license_key_instance_id": license_key_instance_id,
                },
                license_validate_params.LicenseValidateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LicenseValidateResponse,
        )


class LicensesResourceWithRawResponse:
    def __init__(self, licenses: LicensesResource) -> None:
        self._licenses = licenses

        self.activate = to_raw_response_wrapper(
            licenses.activate,
        )
        self.deactivate = to_raw_response_wrapper(
            licenses.deactivate,
        )
        self.validate = to_raw_response_wrapper(
            licenses.validate,
        )


class AsyncLicensesResourceWithRawResponse:
    def __init__(self, licenses: AsyncLicensesResource) -> None:
        self._licenses = licenses

        self.activate = async_to_raw_response_wrapper(
            licenses.activate,
        )
        self.deactivate = async_to_raw_response_wrapper(
            licenses.deactivate,
        )
        self.validate = async_to_raw_response_wrapper(
            licenses.validate,
        )


class LicensesResourceWithStreamingResponse:
    def __init__(self, licenses: LicensesResource) -> None:
        self._licenses = licenses

        self.activate = to_streamed_response_wrapper(
            licenses.activate,
        )
        self.deactivate = to_streamed_response_wrapper(
            licenses.deactivate,
        )
        self.validate = to_streamed_response_wrapper(
            licenses.validate,
        )


class AsyncLicensesResourceWithStreamingResponse:
    def __init__(self, licenses: AsyncLicensesResource) -> None:
        self._licenses = licenses

        self.activate = async_to_streamed_response_wrapper(
            licenses.activate,
        )
        self.deactivate = async_to_streamed_response_wrapper(
            licenses.deactivate,
        )
        self.validate = async_to_streamed_response_wrapper(
            licenses.validate,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Generic, TypeVar, Optional, cast
from typing_extensions import override

from ._base_client import BasePage, PageInfo, BaseSyncPage, BaseAsyncPage

__all__ = ["SyncDefaultPageNumberPagination", "AsyncDefaultPageNumberPagination"]

_T = TypeVar("_T")


class SyncDefaultPageNumberPagination(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    items: List[_T]

    @override
    def _get_page_items(self) -> List[_T]:
        items = self.items
        if not items:
            return []
        return items

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        last_page = cast("int | None", self._options.params.get("page_number")) or 1

        return PageInfo(params={"page_number": last_page + 1})


class AsyncDefaultPageNumberPagination(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    items: List[_T]

    @override
    def _get_page_items(self) -> List[_T]:
        items = self.items
        if not items:
            return []
        return items

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        last_page = cast("int | None", self._options.params.get("page_number")) or 1

        return PageInfo(params={"page_number": last_page + 1})



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from .._models import BaseModel

__all__ = ["LicenseKeyInstance"]


class LicenseKeyInstance(BaseModel):
    id: str

    business_id: str

    created_at: datetime

    license_key_id: str

    name: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..types import refund_list_params, refund_create_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultPageNumberPagination, AsyncDefaultPageNumberPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.refund import Refund

__all__ = ["RefundsResource", "AsyncRefundsResource"]


class RefundsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RefundsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return RefundsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RefundsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return RefundsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        payment_id: str,
        items: Optional[Iterable[refund_create_params.Item]] | NotGiven = NOT_GIVEN,
        reason: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Refund:
        """
        Args:
          payment_id: The unique identifier of the payment to be refunded.

          items: Partially Refund an Individual Item

          reason: The reason for the refund, if any. Maximum length is 3000 characters. Optional.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/refunds",
            body=maybe_transform(
                {
                    "payment_id": payment_id,
                    "items": items,
                    "reason": reason,
                },
                refund_create_params.RefundCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Refund,
        )

    def retrieve(
        self,
        refund_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Refund:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not refund_id:
            raise ValueError(f"Expected a non-empty value for `refund_id` but received {refund_id!r}")
        return self._get(
            f"/refunds/{refund_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Refund,
        )

    def list(
        self,
        *,
        created_at_gte: Union[str, datetime] | NotGiven = NOT_GIVEN,
        created_at_lte: Union[str, datetime] | NotGiven = NOT_GIVEN,
        customer_id: str | NotGiven = NOT_GIVEN,
        page_number: int | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        status: Literal["succeeded", "failed", "pending", "review"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncDefaultPageNumberPagination[Refund]:
        """
        Args:
          created_at_gte: Get events after this created time

          created_at_lte: Get events created before this time

          customer_id: Filter by customer_id

          page_number: Page number default is 0

          page_size: Page size default is 10 max is 100

          status: Filter by status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/refunds",
            page=SyncDefaultPageNumberPagination[Refund],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "created_at_gte": created_at_gte,
                        "created_at_lte": created_at_lte,
                        "customer_id": customer_id,
                        "page_number": page_number,
                        "page_size": page_size,
                        "status": status,
                    },
                    refund_list_params.RefundListParams,
                ),
            ),
            model=Refund,
        )


class AsyncRefundsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRefundsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRefundsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRefundsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return AsyncRefundsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        payment_id: str,
        items: Optional[Iterable[refund_create_params.Item]] | NotGiven = NOT_GIVEN,
        reason: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Refund:
        """
        Args:
          payment_id: The unique identifier of the payment to be refunded.

          items: Partially Refund an Individual Item

          reason: The reason for the refund, if any. Maximum length is 3000 characters. Optional.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/refunds",
            body=await async_maybe_transform(
                {
                    "payment_id": payment_id,
                    "items": items,
                    "reason": reason,
                },
                refund_create_params.RefundCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Refund,
        )

    async def retrieve(
        self,
        refund_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Refund:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not refund_id:
            raise ValueError(f"Expected a non-empty value for `refund_id` but received {refund_id!r}")
        return await self._get(
            f"/refunds/{refund_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Refund,
        )

    def list(
        self,
        *,
        created_at_gte: Union[str, datetime] | NotGiven = NOT_GIVEN,
        created_at_lte: Union[str, datetime] | NotGiven = NOT_GIVEN,
        customer_id: str | NotGiven = NOT_GIVEN,
        page_number: int | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        status: Literal["succeeded", "failed", "pending", "review"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[Refund, AsyncDefaultPageNumberPagination[Refund]]:
        """
        Args:
          created_at_gte: Get events after this created time

          created_at_lte: Get events created before this time

          customer_id: Filter by customer_id

          page_number: Page number default is 0

          page_size: Page size default is 10 max is 100

          status: Filter by status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/refunds",
            page=AsyncDefaultPageNumberPagination[Refund],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "created_at_gte": created_at_gte,
                        "created_at_lte": created_at_lte,
                        "customer_id": customer_id,
                        "page_number": page_number,
                        "page_size": page_size,
                        "status": status,
                    },
                    refund_list_params.RefundListParams,
                ),
            ),
            model=Refund,
        )


class RefundsResourceWithRawResponse:
    def __init__(self, refunds: RefundsResource) -> None:
        self._refunds = refunds

        self.create = to_raw_response_wrapper(
            refunds.create,
        )
        self.retrieve = to_raw_response_wrapper(
            refunds.retrieve,
        )
        self.list = to_raw_response_wrapper(
            refunds.list,
        )


class AsyncRefundsResourceWithRawResponse:
    def __init__(self, refunds: AsyncRefundsResource) -> None:
        self._refunds = refunds

        self.create = async_to_raw_response_wrapper(
            refunds.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            refunds.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            refunds.list,
        )


class RefundsResourceWithStreamingResponse:
    def __init__(self, refunds: RefundsResource) -> None:
        self._refunds = refunds

        self.create = to_streamed_response_wrapper(
            refunds.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            refunds.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            refunds.list,
        )


class AsyncRefundsResourceWithStreamingResponse:
    def __init__(self, refunds: AsyncRefundsResource) -> None:
        self._refunds = refunds

        self.create = async_to_streamed_response_wrapper(
            refunds.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            refunds.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            refunds.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions
from typing import Union
from datetime import datetime

import httpx

from ..types import webhook_event_list_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultPageNumberPagination, AsyncDefaultPageNumberPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.webhook_event import WebhookEvent

__all__ = ["WebhookEventsResource", "AsyncWebhookEventsResource"]


class WebhookEventsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> WebhookEventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return WebhookEventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WebhookEventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return WebhookEventsResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    def retrieve(
        self,
        webhook_event_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WebhookEvent:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not webhook_event_id:
            raise ValueError(f"Expected a non-empty value for `webhook_event_id` but received {webhook_event_id!r}")
        return self._get(
            f"/webhook_events/{webhook_event_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookEvent,
        )

    def list(
        self,
        *,
        created_at_gte: Union[str, datetime] | NotGiven = NOT_GIVEN,
        created_at_lte: Union[str, datetime] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        object_id: str | NotGiven = NOT_GIVEN,
        page_number: int | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        webhook_event_id: str | NotGiven = NOT_GIVEN,
        webhook_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncDefaultPageNumberPagination[WebhookEvent]:
        """
        Args:
          created_at_gte: Get events after this created time

          created_at_lte: Get events created before this time

          limit: Min : 1, Max : 100, default 10

          object_id: Get events history of a specific object like payment/subscription/refund/dispute

          page_number: Page number default is 0

          page_size: Page size default is 10 max is 100

          webhook_event_id: Filter by webhook event id

          webhook_id: Filter by webhook destination

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/webhook_events",
            page=SyncDefaultPageNumberPagination[WebhookEvent],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "created_at_gte": created_at_gte,
                        "created_at_lte": created_at_lte,
                        "limit": limit,
                        "object_id": object_id,
                        "page_number": page_number,
                        "page_size": page_size,
                        "webhook_event_id": webhook_event_id,
                        "webhook_id": webhook_id,
                    },
                    webhook_event_list_params.WebhookEventListParams,
                ),
            ),
            model=WebhookEvent,
        )


class AsyncWebhookEventsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncWebhookEventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return AsyncWebhookEventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWebhookEventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return AsyncWebhookEventsResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    async def retrieve(
        self,
        webhook_event_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> WebhookEvent:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not webhook_event_id:
            raise ValueError(f"Expected a non-empty value for `webhook_event_id` but received {webhook_event_id!r}")
        return await self._get(
            f"/webhook_events/{webhook_event_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookEvent,
        )

    def list(
        self,
        *,
        created_at_gte: Union[str, datetime] | NotGiven = NOT_GIVEN,
        created_at_lte: Union[str, datetime] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        object_id: str | NotGiven = NOT_GIVEN,
        page_number: int | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        webhook_event_id: str | NotGiven = NOT_GIVEN,
        webhook_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[WebhookEvent, AsyncDefaultPageNumberPagination[WebhookEvent]]:
        """
        Args:
          created_at_gte: Get events after this created time

          created_at_lte: Get events created before this time

          limit: Min : 1, Max : 100, default 10

          object_id: Get events history of a specific object like payment/subscription/refund/dispute

          page_number: Page number default is 0

          page_size: Page size default is 10 max is 100

          webhook_event_id: Filter by webhook event id

          webhook_id: Filter by webhook destination

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/webhook_events",
            page=AsyncDefaultPageNumberPagination[WebhookEvent],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "created_at_gte": created_at_gte,
                        "created_at_lte": created_at_lte,
                        "limit": limit,
                        "object_id": object_id,
                        "page_number": page_number,
                        "page_size": page_size,
                        "webhook_event_id": webhook_event_id,
                        "webhook_id": webhook_id,
                    },
                    webhook_event_list_params.WebhookEventListParams,
                ),
            ),
            model=WebhookEvent,
        )


class WebhookEventsResourceWithRawResponse:
    def __init__(self, webhook_events: WebhookEventsResource) -> None:
        self._webhook_events = webhook_events

        self.retrieve = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                webhook_events.retrieve  # pyright: ignore[reportDeprecated],
            )
        )
        self.list = to_raw_response_wrapper(
            webhook_events.list,
        )


class AsyncWebhookEventsResourceWithRawResponse:
    def __init__(self, webhook_events: AsyncWebhookEventsResource) -> None:
        self._webhook_events = webhook_events

        self.retrieve = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                webhook_events.retrieve  # pyright: ignore[reportDeprecated],
            )
        )
        self.list = async_to_raw_response_wrapper(
            webhook_events.list,
        )


class WebhookEventsResourceWithStreamingResponse:
    def __init__(self, webhook_events: WebhookEventsResource) -> None:
        self._webhook_events = webhook_events

        self.retrieve = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                webhook_events.retrieve  # pyright: ignore[reportDeprecated],
            )
        )
        self.list = to_streamed_response_wrapper(
            webhook_events.list,
        )


class AsyncWebhookEventsResourceWithStreamingResponse:
    def __init__(self, webhook_events: AsyncWebhookEventsResource) -> None:
        self._webhook_events = webhook_events

        self.retrieve = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                webhook_events.retrieve  # pyright: ignore[reportDeprecated],
            )
        )
        self.list = async_to_streamed_response_wrapper(
            webhook_events.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

from .currency import Currency
from .tax_category import TaxCategory

__all__ = ["AddonCreateParams"]


class AddonCreateParams(TypedDict, total=False):
    currency: Required[Currency]
    """The currency of the Addon"""

    name: Required[str]
    """Name of the Addon"""

    price: Required[int]
    """Amount of the addon"""

    tax_category: Required[TaxCategory]
    """Tax category applied to this Addon"""

    description: Optional[str]
    """Optional description of the Addon"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Iterable, Optional
from typing_extensions import Literal, Required, TypedDict

from .currency import Currency
from .billing_address_param import BillingAddressParam
from .customer_request_param import CustomerRequestParam

__all__ = ["SubscriptionCreateParams", "Addon", "OnDemand"]


class SubscriptionCreateParams(TypedDict, total=False):
    billing: Required[BillingAddressParam]
    """Billing address information for the subscription"""

    customer: Required[CustomerRequestParam]
    """Customer details for the subscription"""

    product_id: Required[str]
    """Unique identifier of the product to subscribe to"""

    quantity: Required[int]
    """Number of units to subscribe for. Must be at least 1."""

    addons: Optional[Iterable[Addon]]
    """Attach addons to this subscription"""

    allowed_payment_method_types: Optional[
        List[
            Literal[
                "credit",
                "debit",
                "upi_collect",
                "upi_intent",
                "apple_pay",
                "cashapp",
                "google_pay",
                "multibanco",
                "bancontact_card",
                "eps",
                "ideal",
                "przelewy24",
                "affirm",
                "klarna",
                "sepa",
                "ach",
                "amazon_pay",
                "afterpay_clearpay",
            ]
        ]
    ]
    """List of payment methods allowed during checkout.

    Customers will **never** see payment methods that are **not** in this list.
    However, adding a method here **does not guarantee** customers will see it.
    Availability still depends on other factors (e.g., customer location, merchant
    settings).
    """

    billing_currency: Optional[Currency]
    """
    Fix the currency in which the end customer is billed. If Dodo Payments cannot
    support that currency for this transaction, it will not proceed
    """

    discount_code: Optional[str]
    """Discount Code to apply to the subscription"""

    metadata: Dict[str, str]
    """Additional metadata for the subscription Defaults to empty if not specified"""

    on_demand: Optional[OnDemand]

    payment_link: Optional[bool]
    """If true, generates a payment link. Defaults to false if not specified."""

    return_url: Optional[str]
    """Optional URL to redirect after successful subscription creation"""

    show_saved_payment_methods: bool
    """Display saved payment methods of a returning customer False by default"""

    tax_id: Optional[str]
    """Tax ID in case the payment is B2B.

    If tax id validation fails the payment creation will fail
    """

    trial_period_days: Optional[int]
    """
    Optional trial period in days If specified, this value overrides the trial
    period set in the product's price Must be between 0 and 10000 days
    """


class Addon(TypedDict, total=False):
    addon_id: Required[str]

    quantity: Required[int]


class OnDemand(TypedDict, total=False):
    mandate_only: Required[bool]
    """
    If set as True, does not perform any charge and only authorizes payment method
    details for future use.
    """

    adaptive_currency_fees_inclusive: Optional[bool]
    """
    Whether adaptive currency fees should be included in the product_price (true) or
    added on top (false). This field is ignored if adaptive pricing is not enabled
    for the business.
    """

    product_currency: Optional[Currency]
    """Optional currency of the product price.

    If not specified, defaults to the currency of the product.
    """

    product_price: Optional[int]
    """
    Product price for the initial charge to customer If not specified the stored
    price of the product will be used Represented in the lowest denomination of the
    currency (e.g., cents for USD). For example, to charge $1.00, pass `100`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["OneTimeProductCartItem"]


class OneTimeProductCartItem(BaseModel):
    product_id: str

    quantity: int

    amount: Optional[int] = None
    """Amount the customer pays if pay_what_you_want is enabled.

    If disabled then amount will be ignored Represented in the lowest denomination
    of the currency (e.g., cents for USD). For example, to charge $1.00, pass `100`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["Customer"]


class Customer(BaseModel):
    business_id: str

    created_at: datetime

    customer_id: str

    email: str

    name: str

    phone_number: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["WebhookEvent"]


class WebhookEvent(BaseModel):
    business_id: str

    created_at: datetime

    event_id: str

    event_type: str

    object_id: str

    latest_attempted_at: Optional[datetime] = None

    request: Optional[str] = None

    response: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.customers import customer_portal_create_params
from ...types.customer_portal_session import CustomerPortalSession

__all__ = ["CustomerPortalResource", "AsyncCustomerPortalResource"]


class CustomerPortalResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CustomerPortalResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return CustomerPortalResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CustomerPortalResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return CustomerPortalResourceWithStreamingResponse(self)

    def create(
        self,
        customer_id: str,
        *,
        send_email: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CustomerPortalSession:
        """
        Args:
          send_email: If true, will send link to user.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return self._post(
            f"/customers/{customer_id}/customer-portal/session",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"send_email": send_email}, customer_portal_create_params.CustomerPortalCreateParams
                ),
            ),
            cast_to=CustomerPortalSession,
        )


class AsyncCustomerPortalResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCustomerPortalResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCustomerPortalResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCustomerPortalResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return AsyncCustomerPortalResourceWithStreamingResponse(self)

    async def create(
        self,
        customer_id: str,
        *,
        send_email: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CustomerPortalSession:
        """
        Args:
          send_email: If true, will send link to user.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return await self._post(
            f"/customers/{customer_id}/customer-portal/session",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"send_email": send_email}, customer_portal_create_params.CustomerPortalCreateParams
                ),
            ),
            cast_to=CustomerPortalSession,
        )


class CustomerPortalResourceWithRawResponse:
    def __init__(self, customer_portal: CustomerPortalResource) -> None:
        self._customer_portal = customer_portal

        self.create = to_raw_response_wrapper(
            customer_portal.create,
        )


class AsyncCustomerPortalResourceWithRawResponse:
    def __init__(self, customer_portal: AsyncCustomerPortalResource) -> None:
        self._customer_portal = customer_portal

        self.create = async_to_raw_response_wrapper(
            customer_portal.create,
        )


class CustomerPortalResourceWithStreamingResponse:
    def __init__(self, customer_portal: CustomerPortalResource) -> None:
        self._customer_portal = customer_portal

        self.create = to_streamed_response_wrapper(
            customer_portal.create,
        )


class AsyncCustomerPortalResourceWithStreamingResponse:
    def __init__(self, customer_portal: AsyncCustomerPortalResource) -> None:
        self._customer_portal = customer_portal

        self.create = async_to_streamed_response_wrapper(
            customer_portal.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Required, TypedDict

from .currency import Currency

__all__ = ["SubscriptionChargeParams"]


class SubscriptionChargeParams(TypedDict, total=False):
    product_price: Required[int]
    """The product price.

    Represented in the lowest denomination of the currency (e.g., cents for USD).
    For example, to charge $1.00, pass `100`.
    """

    adaptive_currency_fees_inclusive: Optional[bool]
    """
    Whether adaptive currency fees should be included in the product_price (true) or
    added on top (false). This field is ignored if adaptive pricing is not enabled
    for the business.
    """

    metadata: Optional[Dict[str, str]]
    """Metadata for the payment.

    If not passed, the metadata of the subscription will be taken
    """

    product_currency: Optional[Currency]
    """Optional currency of the product price.

    If not specified, defaults to the currency of the product.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["LicenseValidateResponse"]


class LicenseValidateResponse(BaseModel):
    valid: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["SubscriptionStatus"]

SubscriptionStatus: TypeAlias = Literal["pending", "active", "on_hold", "paused", "cancelled", "failed", "expired"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel
from .time_interval import TimeInterval

__all__ = ["LicenseKeyDuration"]


class LicenseKeyDuration(BaseModel):
    count: int

    interval: TimeInterval



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from dodopayments import DodoPayments, AsyncDodoPayments
from dodopayments.types import (
    Discount,
)
from dodopayments._utils import parse_datetime
from dodopayments.pagination import SyncDefaultPageNumberPagination, AsyncDefaultPageNumberPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDiscounts:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: DodoPayments) -> None:
        discount = client.discounts.create(
            amount=0,
            type="percentage",
        )
        assert_matches_type(Discount, discount, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: DodoPayments) -> None:
        discount = client.discounts.create(
            amount=0,
            type="percentage",
            code="code",
            expires_at=parse_datetime("2019-12-27T18:11:19.117Z"),
            name="name",
            restricted_to=["string"],
            usage_limit=0,
        )
        assert_matches_type(Discount, discount, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: DodoPayments) -> None:
        response = client.discounts.with_raw_response.create(
            amount=0,
            type="percentage",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        discount = response.parse()
        assert_matches_type(Discount, discount, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: DodoPayments) -> None:
        with client.discounts.with_streaming_response.create(
            amount=0,
            type="percentage",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            discount = response.parse()
            assert_matches_type(Discount, discount, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: DodoPayments) -> None:
        discount = client.discounts.retrieve(
            "discount_id",
        )
        assert_matches_type(Discount, discount, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: DodoPayments) -> None:
        response = client.discounts.with_raw_response.retrieve(
            "discount_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        discount = response.parse()
        assert_matches_type(Discount, discount, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: DodoPayments) -> None:
        with client.discounts.with_streaming_response.retrieve(
            "discount_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            discount = response.parse()
            assert_matches_type(Discount, discount, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: DodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `discount_id` but received ''"):
            client.discounts.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: DodoPayments) -> None:
        discount = client.discounts.update(
            discount_id="discount_id",
        )
        assert_matches_type(Discount, discount, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: DodoPayments) -> None:
        discount = client.discounts.update(
            discount_id="discount_id",
            amount=0,
            code="code",
            expires_at=parse_datetime("2019-12-27T18:11:19.117Z"),
            name="name",
            restricted_to=["string"],
            type="percentage",
            usage_limit=0,
        )
        assert_matches_type(Discount, discount, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: DodoPayments) -> None:
        response = client.discounts.with_raw_response.update(
            discount_id="discount_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        discount = response.parse()
        assert_matches_type(Discount, discount, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: DodoPayments) -> None:
        with client.discounts.with_streaming_response.update(
            discount_id="discount_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            discount = response.parse()
            assert_matches_type(Discount, discount, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: DodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `discount_id` but received ''"):
            client.discounts.with_raw_response.update(
                discount_id="",
            )

    @parametrize
    def test_method_list(self, client: DodoPayments) -> None:
        discount = client.discounts.list()
        assert_matches_type(SyncDefaultPageNumberPagination[Discount], discount, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: DodoPayments) -> None:
        discount = client.discounts.list(
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultPageNumberPagination[Discount], discount, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: DodoPayments) -> None:
        response = client.discounts.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        discount = response.parse()
        assert_matches_type(SyncDefaultPageNumberPagination[Discount], discount, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: DodoPayments) -> None:
        with client.discounts.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            discount = response.parse()
            assert_matches_type(SyncDefaultPageNumberPagination[Discount], discount, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: DodoPayments) -> None:
        discount = client.discounts.delete(
            "discount_id",
        )
        assert discount is None

    @parametrize
    def test_raw_response_delete(self, client: DodoPayments) -> None:
        response = client.discounts.with_raw_response.delete(
            "discount_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        discount = response.parse()
        assert discount is None

    @parametrize
    def test_streaming_response_delete(self, client: DodoPayments) -> None:
        with client.discounts.with_streaming_response.delete(
            "discount_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            discount = response.parse()
            assert discount is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: DodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `discount_id` but received ''"):
            client.discounts.with_raw_response.delete(
                "",
            )


class TestAsyncDiscounts:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncDodoPayments) -> None:
        discount = await async_client.discounts.create(
            amount=0,
            type="percentage",
        )
        assert_matches_type(Discount, discount, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncDodoPayments) -> None:
        discount = await async_client.discounts.create(
            amount=0,
            type="percentage",
            code="code",
            expires_at=parse_datetime("2019-12-27T18:11:19.117Z"),
            name="name",
            restricted_to=["string"],
            usage_limit=0,
        )
        assert_matches_type(Discount, discount, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.discounts.with_raw_response.create(
            amount=0,
            type="percentage",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        discount = await response.parse()
        assert_matches_type(Discount, discount, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.discounts.with_streaming_response.create(
            amount=0,
            type="percentage",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            discount = await response.parse()
            assert_matches_type(Discount, discount, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncDodoPayments) -> None:
        discount = await async_client.discounts.retrieve(
            "discount_id",
        )
        assert_matches_type(Discount, discount, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.discounts.with_raw_response.retrieve(
            "discount_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        discount = await response.parse()
        assert_matches_type(Discount, discount, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.discounts.with_streaming_response.retrieve(
            "discount_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            discount = await response.parse()
            assert_matches_type(Discount, discount, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncDodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `discount_id` but received ''"):
            await async_client.discounts.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncDodoPayments) -> None:
        discount = await async_client.discounts.update(
            discount_id="discount_id",
        )
        assert_matches_type(Discount, discount, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncDodoPayments) -> None:
        discount = await async_client.discounts.update(
            discount_id="discount_id",
            amount=0,
            code="code",
            expires_at=parse_datetime("2019-12-27T18:11:19.117Z"),
            name="name",
            restricted_to=["string"],
            type="percentage",
            usage_limit=0,
        )
        assert_matches_type(Discount, discount, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.discounts.with_raw_response.update(
            discount_id="discount_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        discount = await response.parse()
        assert_matches_type(Discount, discount, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.discounts.with_streaming_response.update(
            discount_id="discount_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            discount = await response.parse()
            assert_matches_type(Discount, discount, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncDodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `discount_id` but received ''"):
            await async_client.discounts.with_raw_response.update(
                discount_id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncDodoPayments) -> None:
        discount = await async_client.discounts.list()
        assert_matches_type(AsyncDefaultPageNumberPagination[Discount], discount, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncDodoPayments) -> None:
        discount = await async_client.discounts.list(
            page_number=0,
            page_size=0,
        )
        assert_matches_type(AsyncDefaultPageNumberPagination[Discount], discount, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.discounts.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        discount = await response.parse()
        assert_matches_type(AsyncDefaultPageNumberPagination[Discount], discount, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.discounts.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            discount = await response.parse()
            assert_matches_type(AsyncDefaultPageNumberPagination[Discount], discount, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncDodoPayments) -> None:
        discount = await async_client.discounts.delete(
            "discount_id",
        )
        assert discount is None

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.discounts.with_raw_response.delete(
            "discount_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        discount = await response.parse()
        assert discount is None

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.discounts.with_streaming_response.delete(
            "discount_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            discount = await response.parse()
            assert discount is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncDodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `discount_id` but received ''"):
            await async_client.discounts.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .currency import Currency

__all__ = ["PayoutListResponse"]


class PayoutListResponse(BaseModel):
    amount: int
    """The total amount of the payout."""

    business_id: str
    """The unique identifier of the business associated with the payout."""

    chargebacks: int
    """The total value of chargebacks associated with the payout."""

    created_at: datetime
    """The timestamp when the payout was created, in UTC."""

    currency: Currency
    """The currency of the payout, represented as an ISO 4217 currency code."""

    fee: int
    """The fee charged for processing the payout."""

    payment_method: str
    """The payment method used for the payout (e.g., bank transfer, card, etc.)."""

    payout_id: str
    """The unique identifier of the payout."""

    refunds: int
    """The total value of refunds associated with the payout."""

    status: Literal["not_initiated", "in_progress", "on_hold", "failed", "success"]
    """The current status of the payout."""

    tax: int
    """The tax applied to the payout."""

    updated_at: datetime
    """The timestamp when the payout was last updated, in UTC."""

    name: Optional[str] = None
    """The name of the payout recipient or purpose."""

    payout_document_url: Optional[str] = None
    """The URL of the document associated with the payout."""

    remarks: Optional[str] = None
    """Any additional remarks or notes associated with the payout."""
